==> jxdl-3.3.5.tgz: package/index.js <==
const version = 'mrtl-joox@3.3.5';
const versitonText = version + ' copyright by Mortal Soft.';
const path = require('path');
const request = require('request');
const colors = require('colors');
const async = require('async');
const Utils = require('./utils');
const cheerio = require('cheerio');
const fs = require('fs');
const url = require('url');

var argv = require('minimist')(process.argv.slice(2));

if (argv.version || argv.v) {
  console.log(versitonText);
} else if (argv.example || argv.e) {
  fs.readFile(__dirname + '/utils/example.txt', (err, data) => {
    if (err) console.log('File utils/example.txt NOT FOUND');
    console.log();
    console.log(data.toString());
    console.log('\n'+versitonText+'\n');
  });
} else if (argv.help || argv.h) {
  fs.readFile(__dirname + '/utils/help.txt', (err, data) => {
    if (err) console.log('File utils/help.txt NOT FOUND');
    console.log();
    console.log(data.toString());
    console.log('\n'+versitonText+'\n');
  });
} else {

  var type = argv.s || argv.singer ? 'singer' : argv.a || argv.album ? 'album' : argv.l || argv.list ? 'playlist' : argv.t || argv.trend ? 'trend' : argv.u || argv.ulist ? 'userlist' : 'not found';
  var id = argv.s || argv.singer || argv.a || argv.album || argv.l || argv.list || argv.t || argv.trend || argv.u || argv.ulist;
  var country = argv.c || argv.country;
  var qlt = argv.q || argv.quality || '128';
  var task = [];
  var reqs = type == 'album' ? Utils.reqAlbum(id, country) : type == 'playlist' ? Utils.reqList(id, country) : Utils.reqUList(id, country);
  var skip = parseInt(''+argv.k) || parseInt(''+argv.skip) || 0;
  Utils.topDir = argv.d || argv.dir || process.cwd();
  Utils.topDir = Utils.topDir === true ? process.cwd() : Utils.topDir;

  console.log('Request ' + type + ': ' + id);
  console.log();

  if (type == 'trend') {
    console.log('Quality: ' + (qlt == 'm4a' ? qlt : qlt + ' kbps'));
  
    reqs = Utils.reqTop(0, 99, id);
    request(reqs, (err, res, body) => {
      if (err) console.log(err.red);
      var listDetails = JSON.parse(body);
      var songlist = listDetails.songlist;

      Utils.prepareFolder(id);

      for (var index = skip; index < songlist.length; index++) {
        var sl = songlist[index];
        var conArts = sl.artist;
        var songReq = Utils.reqSong(sl.songid);

        task.push(downloadReq(index, songReq, id, conArts));
      }

      async.series(task, (err, res) => {
        if (err) console.log(err);
        console.log('\nFinished!');
        console.log('Check on "' + path.join(Utils.topDir, 'downloads') + '" directory');
      });
    });
  } else if (type == 'singer') {
    console.log('Quality: ' + (qlt == 'm4a' ? qlt : qlt + ' kbps'));
  
    reqs = Utils.reqArtist(id, country);
    request(reqs, (err, res, body) => {
      var $ = cheerio.load(body);

      var artistName = $('h1.artistName').text();
      Utils.prepareFolder(artistName);

      $('div.ListItem').each((i, elem) => {
        if (i >= skip) {
          var sLink = $(elem).children('.song').children('h3').children('a').attr('href')
          var sLinkSp = sLink.trim().split('/')
          
          var songReq = Utils.reqSong(sLinkSp[sLinkSp.length-1]);

          var albumName = $(elem).children('.album').children('a').attr('title')
          
          task.push(downloadReq(i, songReq, artistName, artistName));
        }
      });

      async.series(task, (err, res) => {
        if (err) console.log(err);
        console.log('\nFinished!');
        console.log('Check on "' + path.join(Utils.topDir, 'downloads') + '" directory');
      });
    });
  } else if (type == 'album' || type == 'playlist' || type == 'userlist') {
    console.log('Quality: ' + (qlt == 'm4a' ? qlt : qlt + ' kbps'));
  
    request(reqs, (err, res, body) => {
      if (body) {
        var $ = cheerio.load(body);
        
        var albumName = $('.AlbumInfo').children('.body').children('h1').attr('title');

        console.log(albumName);

        Utils.prepareFolder(albumName);

        $('div.ListItem').each((i, elem) => {
          if (i >= skip) {
            var sLink = $(elem).children('.song').children('h3').children('a').attr('href')
            var sLinkSp = sLink.trim().split('/')
            
            var songReq = Utils.reqSong(sLinkSp[sLinkSp.length-1]);

            var aLink = $(elem).children('.artists').children('span').children('a').attr('title')

            var conArts = [];
            var temp = aLink.split('; ');
            for (var index = 0; index < temp.length; index++) {
              conArts[index] = temp[index].split(':')[0];
            }

            task.push(downloadReq(i, songReq, albumName, conArts.join(';')));
          }
        });

        async.series(task, (err, res) => {
          if (err) console.log(err);
          console.log('\nFinished!');
          console.log('Check on "' + path.join(Utils.topDir, 'downloads') + '" directory');
        });
      } else {
        console.log('Request Error');
      }
    });
  } else {
    console.log('Not Valid Argument, type -h or --help for more information.');
  }

  var downloadReq = (i, songReq, albumName, conArts) => {
    var funOut = (cb) => {
      request(songReq, (err, res, body) => {
        if (body) {
          var songDetails = Utils.parseSongDetails(body);
          var fileUrl = (qlt == 'm4a' && songDetails.m4aUrl) ? songDetails.m4aUrl : qlt == '320' && songDetails.r320Url ? songDetails.r320Url : qlt == '192' && songDetails.r192Url ? songDetails.r192Url : songDetails.mp3Url;
          var map = JSON.parse(songDetails.kbps_map);
          var fileSize = qlt == 'm4a' && songDetails.m4aUrl ? map['96'] : qlt == '320' && songDetails.r320Url ? map['320'] : qlt == '192' && songDetails.r192Url ? map['192'] : map['128'];
          var ext = qlt == 'm4a' ? '.m4a' : '.mp3';
          var fileName = songDetails.msong + ext;

          if (fileUrl) {
            request.get(fileUrl)
              .on('error', (err) => {
                console.log(fileName + ' ERROR');
                cb(null, fileName + ' ERROR');
              })
              .pipe(Utils.makeStream(fileUrl, fileSize, i, albumName, fileName, {
                trackNumber: ""+(i+1),
                title: songDetails.msong,
                artist: conArts || songDetails.msinger,
                album: songDetails.malbum,
                image: Utils.imageUrl(songDetails.imgSrc)
              }, cb));
            } else {
              console.log(fileName + ': ' + songDetails.msg);
              cb(null, 'ERROR');
            }
        } else {
          console.log(' ERROR');
          cb(null, fileName + ' ERROR');
        }
      });
    };

    return funOut;
  };
}

==> jxdl-3.3.8.tgz: package/index.js <==
const version = 'mrtl-joox@3.3.8';
const versitonText = version + ' copyright by Mortal Soft.';
const path = require('path');
const request = require('request');
const colors = require('colors');
const async = require('async');
const Utils = require('./utils');
const cheerio = require('cheerio');
const fs = require('fs');
const url = require('url');

var argv = require('minimist')(process.argv.slice(2));

if (argv.version || argv.v) {
  console.log(versitonText);
} else if (argv.example || argv.e) {
  fs.readFile(__dirname + '/utils/example.txt', (err, data) => {
    if (err) console.log('File utils/example.txt NOT FOUND');
    console.log();
    console.log(data.toString());
    console.log('\n'+versitonText+'\n');
  });
} else if (argv.help || argv.h) {
  fs.readFile(__dirname + '/utils/help.txt', (err, data) => {
    if (err) console.log('File utils/help.txt NOT FOUND');
    console.log();
    console.log(data.toString());
    console.log('\n'+versitonText+'\n');
  });
} else {

  var type = argv.s || argv.singer ? 'singer' : argv.a || argv.album ? 'album' : argv.l || argv.list ? 'playlist' : argv.t || argv.trend ? 'trend' : argv.u || argv.ulist ? 'userlist' : 'not found';
  var id = argv.s || argv.singer || argv.a || argv.album || argv.l || argv.list || argv.t || argv.trend || argv.u || argv.ulist;
  var country = argv.c || argv.country;
  var qlt = argv.q || argv.quality || '128';
  var task = [];
  var reqs = type == 'album' ? Utils.reqAlbum(id, country) : type == 'playlist' ? Utils.reqList(id, country) : Utils.reqUList(id);
  var skip = parseInt(''+argv.k) || parseInt(''+argv.skip) || 0;
  Utils.topDir = argv.d || argv.dir || process.cwd();
  Utils.topDir = Utils.topDir === true ? process.cwd() : Utils.topDir;

  console.log('Request ' + type + ': ' + id);
  console.log();

  if (type == 'trend') {
    console.log('Quality: ' + (qlt == 'm4a' ? qlt : qlt + ' kbps'));
  
    reqs = Utils.reqTop(0, 99, id);
    request(reqs, (err, res, body) => {
      if (err) console.log(err.red);
      var listDetails = JSON.parse(body);
      var songlist = listDetails.songlist;

      Utils.prepareFolder(id);

      for (var index = skip; index < songlist.length; index++) {
        var sl = songlist[index];
        var conArts = sl.artist;
        var songReq = Utils.reqSong(sl.songid);

        task.push(downloadReq(index, songReq, id, conArts));
      }

      async.series(task, (err, res) => {
        if (err) console.log(err);
        console.log('\nFinished!');
        console.log('Check on "' + path.join(Utils.topDir, 'downloads') + '" directory');
      });
    });  
  } else if (type == 'userlist') {
    console.log('Quality: ' + (qlt == 'm4a' ? qlt : qlt + ' kbps'));
  
    request(reqs, (err, res, body) => {
      if (err) console.log(err.red);
      var listDetails = JSON.parse(body);
      var songlist = listDetails.item;
      var name = url.parse(id, true).query.listid;

      Utils.prepareFolder(name);

      for (var index = skip; index < songlist.length; index++) {
        var sl = songlist[index];
        var buf = Buffer.from(sl.singername, 'base64');
        var conArts = buf.toString();
        var songReq = Utils.reqSong(sl.gl);

        task.push(downloadReq(index, songReq, name, conArts));
      }

      async.series(task, (err, res) => { //diff
        if (err) console.log(err);
        console.log('\nFinished!');
        console.log('Check on "' + path.join(Utils.topDir, 'downloads') + '" directory');
      });
    });
  } else if (type == 'singer') {
    console.log('Quality: ' + (qlt == 'm4a' ? qlt : qlt + ' kbps'));
  
    reqs = Utils.reqArtist(id, country);
    request(reqs, (err, res, body) => {
      var $ = cheerio.load(body);

      var artistName = $('h1.artistName').text();
      Utils.prepareFolder(artistName);

      $('div.ListItem').each((i, elem) => {
        if (i >= skip) {
          var sLink = $(elem).children('.song').children('h3').children('a').attr('href')
          var sLinkSp = sLink.trim().split('/')
          
          var songReq = Utils.reqSong(sLinkSp[sLinkSp.length-1]);

          var albumName = $(elem).children('.album').children('a').attr('title')
          
          task.push(downloadReq(i, songReq, artistName, artistName));
        }
      });

      async.series(task, (err, res) => {
        if (err) console.log(err);
        console.log('\nFinished!');
        console.log('Check on "' + path.join(Utils.topDir, 'downloads') + '" directory');
      });
    });
  } else if (type == 'album' || type == 'playlist') {
    console.log('Quality: ' + (qlt == 'm4a' ? qlt : qlt + ' kbps'));
  
    request(reqs, (err, res, body) => {
      if (body) {
        var $ = cheerio.load(body);
        
        var albumName = $('.AlbumInfo').children('.body').children().first().children('span').text();

        console.log(albumName);

        Utils.prepareFolder(albumName);

        $('div.ListItem').each((i, elem) => {
          if (i >= skip) {
            var sLink = $(elem).children('.song').children('h3').children('a').attr('href')
            var sLinkSp = sLink.trim().split('/')
            
            var songReq = Utils.reqSong(sLinkSp[sLinkSp.length-1]);

            var aLink = $(elem).children('.artists').children('span').children('a').attr('title')

            var conArts = [];
            var temp = aLink.split('; ');
            for (var index = 0; index < temp.length; index++) {
              conArts[index] = temp[index].split(':')[0];
            }

            task.push(downloadReq(i, songReq, albumName, conArts.join(';')));
          }
        });

        async.series(task, (err, res) => {
          if (err) console.log(err);
          console.log('\nFinished!');
          console.log('Check on "' + path.join(Utils.topDir, 'downloads') + '" directory');
        });
      } else {
        console.log('Request Error');
      }
    });
  } else {
    console.log('Not Valid Argument, type -h or --help for more information.');
  }

  var downloadReq = (i, songReq, albumName, conArts) => {
    var funOut = (cb) => {
      request(songReq, (err, res, body) => {
        if (body) {
          var songDetails = Utils.parseSongDetails(body);
          var fileUrl = (qlt == 'm4a' && songDetails.m4aUrl) ? songDetails.m4aUrl : qlt == '320' && songDetails.r320Url ? songDetails.r320Url : qlt == '192' && songDetails.r192Url ? songDetails.r192Url : songDetails.mp3Url;
          var map = JSON.parse(songDetails.kbps_map);
          var fileSize = qlt == 'm4a' && songDetails.m4aUrl ? map['96'] : qlt == '320' && songDetails.r320Url ? map['320'] : qlt == '192' && songDetails.r192Url ? map['192'] : map['128'];
          var ext = qlt == 'm4a' ? '.m4a' : '.mp3';
          var fileName = songDetails.msong + ext;

          if (fileUrl) {
            request.get(fileUrl)
              .on('error', (err) => {
                console.log(fileName + ' ERROR');
                cb(null, fileName + ' ERROR');
              })
              .pipe(Utils.makeStream(fileUrl, fileSize, i, albumName, fileName, {
                trackNumber: ""+(i+1),
                title: songDetails.msong,
                artist: conArts || songDetails.msinger,
                album: songDetails.malbum,
                image: Utils.imageUrl(songDetails.imgSrc)
              }, cb));
            } else {
              console.log(fileName + ': ' + songDetails.msg);
              cb(null, 'ERROR');
            }
        } else {
          console.log(' ERROR');
          cb(null, fileName + ' ERROR');
        }
      });
    };

    return funOut;
  };
}
